// Persistence of Vision Ray Tracer Scene Description File
// File: esr_macs.inc
// Vers: 3.5
// Desc: Electrostatic-Repulsion Macros for Even Distribution on Spheres 
// Date: 30.08.03 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: (dd.mm.yy)

//Required Files
// None

//Description
//Given some standard descriptions, the macro will save a file with an array
//of given size, with points evenly distributed (to an accuracy degree dependant on
//the settings) on a sphere.

/*Simply call:
 ESR_Macro(
  Point_Number,
  Radius of Sphere,
  Center of Sphere,
  Maximum Iterations,
  Minimum Force to Reach to stop Iterations,
  Filename (without ending, it will default to add ".esr" by itself),
  Name of array in File,
  Overwrite-Protection-Boolean
  )

Optionally, you may declare R as a seed outside of the macro to influence the
initial random distribution with which the macro starts off.

As a security-option, the macro won't overwrite files itself has generated if
the Overwrite-Boolean is !=0. In this manner, you may protect yourself of
overwriting some other file that took very long to parse (which may
happen with large amounts of points). Instead of overwriting, the macro
will simply add a running number and increase it with every call. Note though
that you'll thus end up including the "first" file all the time, not the
new ones, so check up on that when you're results don't change even though
you're doing all kinds of changes to the setup.

For additional usage with the resulting array, I've supplied Get_Min_Distance,
which takes a 1D-array of Points as input and returns the minimum distance
between any of these points.
*/

//My files sometimes build on each other. Checking if following parameter
//was declared/is set to true avoids my other include-files to include
//an already included file again.
#declare _esr_macs_inc_tnw=1;

#ifndef (ESR_Debug) #declare ESR_Debug=off; #end
#ifndef (R) #declare R=seed(260981); #end

//ESR_Macro
//Parameters:
// 1. Amount of Points
// 2. Radius of Sphere
// 3. Center of Sphere
// 4. Maximum number of iterations (keep somewhere around 250 for good results)
// 5. If this force is underscored, the iterations will stop
// 6. Filename (without Ending) for the Output
// 7. Name or array (in the Output-File)
// 8. Overwrite-Boolean (will not overwrite older files if 0)
//Returns:
// Filename+".esr" with evenly distributed points with given number in array inside
//Notes:
// None
#macro ESR_Macro(_Electron_Amount,_Radius,_Center,_MaxIterations,_MinForce,_Filename,_ArrayName,_Overwrite)
 #local _Array=array[_Electron_Amount];
 //Initial Random spread
 #local A=0;
 #while (A<_Electron_Amount)
  #local _Array[A]=vnormalize(<rand(R)*2-1,rand(R)*2-1,rand(R)*2-1>);
  #local A=A+1;
 #end
 #local _SumForce=2*_MinForce;
 #local _Pow=array[_Electron_Amount];
 //Parse through iterations
 #local _I=0;
 #while (_I<_MaxIterations & _SumForce>=_MinForce)
  #local _SumForce=0;
  //Parse per Electron
  #local A=0;
  #while (A<_Electron_Amount)
   #local _Pow[A]=<0,0,0>;
   //Compare with each other Electron and gather forces
   #local B=0;
   #while (B<_Electron_Amount)
    #if (A!=B) #local _Pow[A]=_Pow[A]+vnormalize(_Array[B]-_Array[A]); #end
    #local B=B+1;
   #end
   #local _SumForce=_SumForce+vlength(vcross(_Pow[A],vnormalize(_Array[A])));
   #local A=A+1;
  #end
  //Finally add force
  #local A=0;
  #while (A<_Electron_Amount)
   #local _Array[A]=vnormalize(_Array[A]-_Pow[A]);
   #local A=A+1;
  #end
  #if (ESR_Debug) #debug concat("Force:",str(_SumForce,1,3),", Iteration:",str(_I,1,0),"\n") #end
  //Run through Iterations
  #local _I=_I+1;
 #end
 //Write Data to Disk
 #local _RealFilename=_Filename;
 //Overwrite-Check
 #if (_Overwrite)
  #local _Val=0;
  #while (file_exists(concat(_RealFilename,".esr")))
   #local _RealFilename=concat(_Filename,str(_Val,1,0));
   #local _Val=_Val+1;
  #end
 #end
 //Finally write to disk
 #fopen _out concat(_RealFilename,".esr") write
 #write (_out, "//Output from Electrostatic-Repulsion Macro, written by Tim Nikias v2.0\n")
 #write (_out, "// Homepage: www.digitaltwilight.de/no_lights\n")
 #write (_out, "#declare ",_ArrayName,"=array[",_Electron_Amount,"] {\n")
 #local A=0;
 #while (A<_Electron_Amount)
  #write (_out, vnormalize(_Array[A])*_Radius+_Center,",")
  #if (mod(A,4)=0 & A!=0) #write (_out, "\n") #end
  #local A=A+1;
 #end
 #write (_out, "}\n")
 #fclose _out
#end

//Get_Min_Distance
//Parameters:
// 1D-Array of Points
//Returns:
// Minimum Distance (float) between any two points
//Notes:
// Brute Force Algorithm...
#macro Get_Min_Distance(_Array_Of_Points)
 #local _Size=dimension_size(_Array_Of_Points,1);
 #local A=0;
 #while (A<_Size)
  #local B=A+1;
  #while (B<_Size)
   #if (A=0 & B=1)
    #local _Min=vlength(_Array_Of_Points[A]-_Array_Of_Points[B]);
   #else
    #local _Dist=vlength(_Array_Of_Points[A]-_Array_Of_Points[B]);
    #if (_Dist < _Min) #local _Min=_Dist; #end
   #end
   #local B=B+1;
  #end
  #local A=A+1;
 #end
 _Min
#end
